@@ -20,17 +20,12 @@
 //  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 //  SOFTWARE.
 
-#include <tiledb/tiledb>
-
-#define STRICT_R_HEADERS
-#include <Rcpp.h>
+#ifndef __libtiledb_h__
+#define __libtiledb_h__
 
 // in inst/include so that Rcpp code generation can use the types for glue code
 #include <tiledb.h>
 
-#ifndef __libtiledb_h__
-#define __libtiledb_h__
-
 // Version
 Rcpp::NumericVector tiledb_version();
 
@@ -47,9 +42,11 @@ Rcpp::CharacterVector tiledb_config_get(Rcpp::XPtr<tiledb::Config> xconfig,
 
 void tiledb_config_dump(Rcpp::XPtr<tiledb::Config> config);
 
-// Conversion helper
+// Conversion helpers
 Rcpp::NumericVector makeNanotime(const std::vector<int64_t>& vec);
 Rcpp::NumericVector makeInteger64(const std::vector<int64_t>& vec);
 int64_t makeScalarInteger64(const double val);
+std::vector<int64_t> getInt64Vector(Rcpp::NumericVector vec);
+bool isInteger64(Rcpp::NumericVector v);
 
 #endif
